/* ===================================== */
/* Header file for DOWNSORT : DOWNSORT.H */
/* ===================================== */

/* used in Maximus CBCS structures */

#ifndef __32BIT__
  #define _HUGE  huge                   /* migration! */
#else
  #define _HUGE                         /* empty */
#endif

#define MAXANAME    10
#define MAXFN       13
#define MAXRCD    2048
#define MAXDESC    (MAXRCD-MAXFN-3)
#define MAXPATH    128
#define MAXTIT      20

#define QMARK       '?'
#define ALPHA       'A'
#define TIMESTAMP   'D'
#define FONT        'F'
#define GROUP       'G'
#define HELP        'H'
#define INCLUDE     'I'
#define KEEPSEQ     'K'
#define LONGLIST    'L'
#define QUIET       'Q'
#define TRUNC       'T'
#define VERBOSE     'V'
#define WRAP        'W'
#define EXCLPRIV    'X'

#define AVA_A      1                 /* ^A  */
#define AVA_B      2                 /* ^B  */
#define AVA_C      3                 /* ^C  */
#define AVA_D      4                 /* ^D  */
#define AVA_E      5                 /* ^E  */
#define AVA_F      6                 /* ^F  */
#define AVA_G      7                 /* ^G  */
#define AVA_H      8                 /* ^H  */
#define AVA_I      9                 /* ^I  */
#define AVA_J      10                /* ^J  */
#define AVA_K      11                /* ^K  */
#define AVA_L      12                /* ^L  */
#define AVA_M      13                /* ^M  */
#define AVA_N      14                /* ^N  */
#define AVA_V      22                /* ^V  */
#define AVA_W      23                /* ^W  */
#define AVA_Y      25                /* ^Y  */

#define DAYS_30     '+'
#define DAYS_7      '*'

#define FONT0       0
#define FONT1       1
#define FONT2       2
#define FONT3       3
#define FONT4       4
#define LINE1       0
#define LINE2       1
#define LINE3       2
#define LINE4       3
#define LINE5       4

/* (foreground) COLORS */

#define O_BLINK   0x80
#define O_BRIGHT  0x08
#define O_LIGHT   0x08

#define O_BLACK   0x00
#define O_BLUE    0x01
#define O_GREEN   0x02
#define O_CYAN    0x03
#define O_RED     0x04
#define O_MAGENTA 0x05
#define O_BROWN   0x06
#define O_GRAY    0x07
#define O_YELLOW  (O_BRIGHT+O_BROWN)
#define O_WHITE   (O_BRIGHT+O_GRAY)

typedef struct _filechain {             /* file information           */
        struct _filechain *next_element;  /* pointer to next element  */
        struct _downpath _HUGE *parea;  /* pointer to area-structure  */
        char   *fdesc;                  /* ptr to file description    */
        char   *fpath;                  /* ptr to file path           */
        ULONG  size;                    /* file size                  */
        ULONG  attr;                    /* attribute                  */
        FDATE  wdate;                   /* file date (last write)     */
        FTIME  wtime;                   /* file time (last write)     */
        FDATE  cdate;                   /* file date (creation)       */
        FTIME  ctime;                   /* file time (creation)       */
        USHORT fseq;                    /* FILES.BBS sequence number  */
        short int priv;                 /* file privilege             */
        unsigned int dl_b:1,            /* unlimited download (bytes) */
                     dl_t:1,            /* unlimited download (time)  */
                     cmt:1;             /* files.bbs comment entry    */
        char   fname[MAXFN];            /* filename                   */
          } FILECHAIN;

typedef struct _downpath {              /* download area information  */
        struct _filechain *newest;      /* pointer to most recent file*/
        ULONG  byte_count;              /* Area byte count area summ. */
        short  int priv;                /* Area Privilege             */
        short  int anum;                /* Area sequence number       */
        USHORT file_count;  /* Area file count area summ. */
        char   name[MAXANAME];          /* MAX Area name              */
        char   ename[MAXANAME];         /* edited right aligned       */
        char   pname[80];               /* Download-pathname          */
        char   filesbbs[80];            /* Path to FILES.BBS          */
        char   adesc[80];               /* Filearea description       */
          } DOWNPATH;

#define P_BBS  0                        /* Symbolic parameter table   */
#define P_FIL  1                        /* entry index values         */
#define P_GBL  2                        /* for the different lists    */
#define P_NEW  3                        /* New acquisitions           */
#define P_ALL  4                        /* All files                  */
#define P_IPF  5                        /* OS/2 1.3 IPF format        */
#define P_DUP  6                        /* Duplicate files            */
#define P_ORP  7                        /* Orphan list                */
#define P_OK   8                        /* OK-file                    */
#define P_IP2  9                        /* OS/2 2.0+ IPF format       */
#define P_EMI 10                        /* Availability list          */
#define P_MAX 10                        /* indicates highest index!   */

typedef struct _listparm {              /* list parameters            */
        short int priv[10];             /* max file priv to list      */
        short int tfont;                /* title font                 */
        unsigned short int max_fil;     /* maximum # of files to list */
        char *incl_fspec;               /* ptr to include-filespec    */
        char longflag;                  /* long OK-file               */
        char sortflag;                  /* file seq. (name/date/keep) */
        char wrapflag;                  /* wrap/truncate              */
        char exclflag;                  /* privilege exclude flag     */
        char listflag;                  /* max list length in number  */
                                        /*   or in days, weeks, months*/
        char name[9];                   /* list filename              */
        char ext[4];                    /* list extension             */
          } LISTPARM;

/* externs for global variables */

extern char   PROGNAME[],PROGDESC[],AUTHOR[],CITY[],PHONE[],FIDO[];
extern char   VERSION,SUBVERS,SUFFIX;
extern char   MAX[],MSG_ARE[],MSG_CFG[],MSG_COA[],MSG_COF[],MSG_FNT[],
              MSG_IEC[],MSG_KWD[],MSG_MEM[],MSG_MX1[],MSG_MX2[],
              MSG_OPA[],MSG_OPI[],MSG_OPO[],MSG_ORP[],MSG_SRT[],MSG_RST[],
              MSG_TRL[],MSG_REC[],MSG_REP[],MSG_UNK[],
              MSG_ZF[],MSG_ZY[],MSG_ZP[],
              OPEN_FAIL[],FILPREFX[],
              BY[],FN[],FS[],AC[],DT[],TM[],SZ[],DS[],FP[],SUMM[],
              MO[],MP[],DF[],NDS[],ORPHAN[],OFFLINE[],
              BAK[],DOT[],EMPTY[],WK[],DAYS[],MONTHS[],WEEKS[],WRITE[];
extern char   today[],list_title[],buf2[],
              filesbbs_path[],areadat_path[],cfg_path[];
extern char   *priv_name[],*pre_title[],*sub_title[],*bot_lines[];
extern char   non_dup_ext[][4], selected_area[][MAXANAME];
extern char   wrap_flag,oper_mode,area_seq;
extern char   strip_ava;
extern int    area_IN_EX;
extern int    MAX_level, ABS_MAX_priv;
extern unsigned int area_total_count, file_total_count;
extern unsigned int max_aname;
extern unsigned short title_font, title_lines[];
extern unsigned long byte_count;
extern FILECHAIN  *first_element;
extern LISTPARM    lp[];
extern COUNTRYCODE c_code;
extern COUNTRYINFO c_info;

